unit Browser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DadRes, Jpeg, ExtCtrls, Menus, ComCtrls;

type
  TfrmBrowser = class(TForm)
    Res: TDadRes;
    MainMenu: TMainMenu;
    Fichier1: TMenuItem;
    Quitter1: TMenuItem;
    Images1: TMenuItem;
    Ajouter1: TMenuItem;
    Supprimer1: TMenuItem;
    Visualiser1: TMenuItem;
    N1: TMenuItem;
    Aide1: TMenuItem;
    Aproposde1: TMenuItem;
    OpenDialog: TOpenDialog;
    ImageList: TImageList;
    ListView: TListView;
    N2: TMenuItem;
    RecrerIcones1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure Ajouter1Click(Sender: TObject);
    procedure Quitter1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure RecrerIcones1Click(Sender: TObject);
    procedure ListViewDblClick(Sender: TObject);
    procedure Aproposde1Click(Sender: TObject);
  private
    Modified : Boolean;
  public
    Images : TStringList;
    Procedure LoadImage(Name : String; Jpg : TJpegImage);
    Procedure MakeIconeFormJpeg(Name : String ; Jpg : TJpegImage ; Bmp : TBitmap);
    procedure AddDescription(Name : String ; Image : Integer);
    procedure ReCreateIcones;
    procedure SaveIcones;
    procedure LoadIcones;
  end;

var
  frmBrowser: TfrmBrowser;

implementation

uses Viewer;

{$R *.DFM}

procedure TfrmBrowser.FormCreate(Sender: TObject);
begin
  Res.FileName:=ExtractFileDir(Application.ExeName)+'\Images.drs';
  Res.IndexFile:=ExtractFileDir(Application.ExeName)+'\Images.idx';
  Res.Open;
  Res.Optimize;
  Images:=TStringList.Create;
  Res.GetNames(Images);
  Caption:=Application.Title;
  LoadIcones;
end;

Procedure TfrmBrowser.LoadImage(Name : String; Jpg : TJpegImage);
var
  Stream : TDadResStream;
Begin
  Stream:=Res.Stream[Name];
  Try
    Jpg.LoadFromStream(Stream);
  Finally
    Stream.Free;
  End;
End;

Procedure TfrmBrowser.MakeIconeFormJpeg(Name : String ; Jpg : TJpegImage ; Bmp : TBitmap);
Var
  Dw,Dh : Real;
  W,H : integer;
  R : TRect;
Begin
  Modified:=True;
  Bmp.Canvas.Brush.Color:=clBtnFace;
  Bmp.Canvas.FillRect(Rect(0,0,Bmp.Width,Bmp.Height));
  Dw:=Jpg.Width/Bmp.Width;
  Dh:=Jpg.Height/Bmp.Height;
  if Dw>Dh then
  Begin
    W:=Bmp.Width;
    H:=Round(Jpg.Height/Dw);
  End Else
  Begin
    W:=Round(Jpg.Width/Dh);
    H:=Bmp.Height;
  End;
  R:=Rect(Bmp.Width div 2 - W div 2,Bmp.Height div 2 - H div 2,Bmp.Width div 2 + W div 2,Bmp.Height div 2 + H div 2);
  Bmp.Canvas.StretchDraw(R,Jpg);
  Frame3D(Bmp.Canvas,R,clBtnHighlight,clBtnShadow,1);
  ImageList.Add(Bmp,nil);
End;

procedure TfrmBrowser.AddDescription(Name : String ; Image : Integer);
Begin
  With ListView.Items.Add do
  Begin
    Caption:=Name;
    ImageIndex:=Image;
  End;
End;

procedure TfrmBrowser.Ajouter1Click(Sender: TObject);
Var
  I : Integer;
  F : String;
  Jpg : TJpegImage;
  Bmp  : TBitmap;
begin
  Bmp:=TBitmap.Create;
  Bmp.Width:=ImageList.Width;
  Bmp.Height:=ImageList.Height;
  Bmp.PixelFormat:=pf24Bit;
  if OpenDialog.Execute and (OpenDialog.Files.Count>0) then
  Try
    For I:=0 to OpenDialog.Files.Count-1 do
    Begin
      F:=ExtractFileName(OpenDialog.Files[I]);
      Caption:=Format('%s - %s (%0.1f %%)',[Application.Title,F,(I+1)/OpenDialog.Files.Count*100]);
      Jpg:=TJpegImage.Create;
      Try
        Try
          Jpg.LoadFromFile(OpenDialog.Files[I]);
          MakeIconeFormJpeg(F,Jpg,Bmp);
          AddDescription(F,ImageList.Count-1);
          Res.AddFile(OpenDialog.Files[I],F,False,0);
          Images.Add(F);
        Finally
          Jpg.Free;
        End;
      Except
      End;
    End;
  Finally
    Res.UpdateFile;
  End;
  Caption:=Application.Title;
end;

procedure TfrmBrowser.Quitter1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmBrowser.ReCreateIcones;
Var
  I : Integer;
  Jpg : TJpegImage;
  Bmp  : TBitmap;
Begin
  ListView.Items.BeginUpdate;
  ImageList.Clear;
  ListView.Items.Clear;

  Bmp:=TBitmap.Create;
  Bmp.Width:=ImageList.Width;
  Bmp.Height:=ImageList.Height;
  Bmp.PixelFormat:=pf24Bit;

  if Images.Count>0 then
  For I:=0 to Images.Count-1 do
  Begin
    Jpg:=TJpegImage.Create;
    Try
      Caption:=Format('%s - %s (%0.1f %%)',[Application.Title,Images[I],(I+1)/Images.Count*100]);
      LoadImage(Images[I],Jpg);
      MakeIconeFormJpeg(Images[I],Jpg,Bmp);
      AddDescription(Images[I],I);
    Finally
      Jpg.Free;
    End;
  End;

  ListView.Items.EndUpdate;
End;

procedure TfrmBrowser.LoadIcones;
Var
  I : Integer;
  Jpg : TJpegImage;
  Bmp : TBitmap;
begin
  ImageList.Clear;
  ListView.Items.Clear;
  if Images.Count>0 then
  Begin
    For I:=0 to Images.Count-1 do
      AddDescription(Images[I],I);
    Bmp:=TBitmap.Create;
    Bmp.Width:=ImageList.Width*4;
    Bmp.Height:=ImageList.Height;
    Bmp.PixelFormat:=pf24Bit;
    Jpg:=TJpegImage.Create;
    Try
      Jpg.LoadFromFile(ExtractFileDir(Application.ExeName)+'\ImageList.jpg');
      Jpg.DIBNeeded;
      For I:=0 to (Images.Count-1) div 4 do
      Begin
        Bmp.Canvas.Draw(0,-(I*ImageList.Height),Jpg);
        ImageList.Add(Bmp,nil);
      End;
    Finally
      Bmp.Free;
      Jpg.Free;
    End;
  End;
  Modified:=False;
End;

procedure TfrmBrowser.SaveIcones;
Var
  Jpg : TJpegImage;
  Bmp : TBitmap;
begin
  if not Modified then Exit;
  if ImageList.Count>0 then
  Begin
    Bmp:=TBitmap.Create;
    Jpg:=TJpegImage.Create;
    Try
      Bmp.Handle:=ImageList.GetImageBitmap;
      Bmp.Dormant;
      Jpg.Assign(Bmp);
      Jpg.CompressionQuality:=50;
      Jpg.Compress;
      Jpg.SaveToFile(ExtractFileDir(Application.ExeName)+'\ImageList.Jpg');
    Finally
      Bmp.Free;
      Jpg.Free;
    End;
  End;
  Modified:=False;
End;

procedure TfrmBrowser.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveIcones;
end;

procedure TfrmBrowser.RecrerIcones1Click(Sender: TObject);
begin
  ReCreateIcones;
  SaveIcones;
end;

procedure TfrmBrowser.ListViewDblClick(Sender: TObject);
begin
  if Assigned(ListView.Selected) then
  With TfrmImageView.Create(nil) do
    ViewImage(Images.IndexOf(ListView.Selected.Caption));
end;

procedure TfrmBrowser.Aproposde1Click(Sender: TObject);
begin
  ShowMessage('Dad''s Image Viewer');
end;

end.
